"""
Ce fichier permet de gerer les categories de formation

Une categorie est rattachées à un et un seul domaine
et est defini par :
- code categorie
- nom categorie
- description
- domaine_id

la clé fonctionnelle est le code
"""
import bson
import pymongo
from pymongo import MongoClient
import json
from bson import ObjectId
import re
from datetime import datetime
import prj_common as mycommon
import secrets
import inspect
import sys, os
import csv
import pandas as pd
from pymongo import ReturnDocument
import GlobalVariable as MYSY_GV
from math import isnan
import GlobalVariable as MYSY_GV
import ela_index_bdd_classes as eibdd
import email_mgt as email
import jinja2
from flask import send_file
from xhtml2pdf import pisa
from email.message import EmailMessage
from email.mime.text import MIMEText
from email import encoders
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders


"""
Ajout d'une categorie
"""

def Add_Class_Categorie(diction):
    try:
        diction = mycommon.strip_dictionary(diction)

        """
        Verification des input acceptés
        """
        field_list = ['token', 'code', 'nom', 'description', 'class_domaine_id']

        incom_keys = diction.keys()
        for val in incom_keys:
            if val not in field_list and val.startswith('my_') is False:
                mycommon.myprint(str(
                    inspect.stack()[0][3]) + "  Le champ  '" + val + "' n'est pas autorisé")
                return False, " Les informations fournies sont incorrectes"

        """
        Verification des champs obligatoires
        """
        field_list_obligatoire = ['token', 'code', 'nom', 'class_domaine_id']

        for val in field_list_obligatoire:
            if val not in diction:
                mycommon.myprint(
                    str(inspect.stack()[0][3]) + " - La valeur '" + val + "' n'est pas presente dans liste ")
                return False, " Les informations fournies sont incorrectes"

        """
        Verification de l'identité et autorisation de l'entité qui
        appelle cette API
        """
        token = ""
        if ("token" in diction.keys()):
            if diction['token']:
                token = diction['token']

        local_status, my_partner = mycommon.Check_Connexion_And_Return_Partner_Data(diction)
        if (local_status is not True):
            return local_status, my_partner


        # Verifier que ce code n'existe pas déjà
        is_existe_class_categorie = MYSY_GV.dbname['class_categorie'].count_documents({'code':str(diction['code']),
                                                                                                       'valide':'1',
                                                                                                       'partner_owner_recid':str(my_partner['recid'])})


        if( is_existe_class_categorie > 0 ):
            mycommon.myprint(
                str(inspect.stack()[0][3]) + " Une catégorie avec le code '" + str(diction['code']) + "' existe déjà ")
            return False, " Une catégorie avec le code '" + str(diction['code']) + "' existe déjà  "


        """
        Verifier que le domaine_id exist et est valide
        """
        is_class_domaine_id_count = MYSY_GV.dbname['class_domaine'].count_documents({'_id':ObjectId(str(diction['class_domaine_id'])),
                                                                                      'valide':'1',
                                                                                      'locked':'0'})

        if (is_class_domaine_id_count != 1) :
            mycommon.myprint(
                str(inspect.stack()[0][3]) + " L'identifiant du domaine de formation n'est pas valide ")
            return False, " L'identifiant du domaine de formation n'est pas valide  "



        new_data = diction
        del diction['token']

        # Initialisation des champs non envoyés à vide
        for val in field_list:
            if val not in diction.keys():
                new_data[str(val)] = ""

        new_data['valide'] = "1"
        new_data['locked'] = "0"

        new_data['date_update'] = str(datetime.now())
        new_data['update_by'] = str(my_partner['_id'])
        new_data['partner_owner_recid'] = str(my_partner['recid'])

        inserted_id = MYSY_GV.dbname['class_categorie'].insert_one(new_data).inserted_id
        if (not inserted_id):
            mycommon.myprint(
                " Impossible de créer la catégorie de formation  (2) ")
            return False, " Impossible de créer la catégorie de formation (2)  "


        return True, " La categorie a été correctement ajoutée"

    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        mycommon.myprint(str(inspect.stack()[0][3]) + " -" + str(e) + " - Line : " + str(exc_tb.tb_lineno))
        return False, " Impossible de créer la catégorie de formation "


"""
Mise à jour d'une categorie de formation
"""

def Update_Class_Categorie(diction):
    try:
        diction = mycommon.strip_dictionary(diction)

        """
        Verification des input acceptés
        """
        field_list =  ['token', '_id', 'code', 'nom', 'description', 'class_domaine_id']

        incom_keys = diction.keys()
        for val in incom_keys:
            if val not in field_list and val.startswith('my_') is False:
                mycommon.myprint(str(
                    inspect.stack()[0][3]) + "  Le champ  '" + val + "' n'est pas autorisé")
                return False, " Les informations fournies sont incorrectes"

        """
        Verification des champs obligatoires
        """
        field_list_obligatoire = ['token', '_id',]

        for val in field_list_obligatoire:
            if val not in diction:
                mycommon.myprint(
                    str(inspect.stack()[0][3]) + " - La valeur '" + val + "' n'est pas presente dans liste ")
                return False, " Les informations fournies sont incorrectes"

        """
        Verification de l'identité et autorisation de l'entité qui
        appelle cette API
        """
        token = ""
        if ("token" in diction.keys()):
            if diction['token']:
                token = diction['token']

        local_status, my_partner = mycommon.Check_Connexion_And_Return_Partner_Data(diction)
        if (local_status is not True):
            return local_status, my_partner

        new_data = diction

        # Verifier que la categorie
        is_existe_cdtion_paiement = MYSY_GV.dbname['class_categorie'].count_documents(
            {'_id': ObjectId(str(diction['_id'])),
             'valide': '1',
             'partner_owner_recid': str(my_partner['recid'])})

        if (is_existe_cdtion_paiement < 0):
            mycommon.myprint(
                str(inspect.stack()[0][3]) + " L'identifiant de la catégorie est invalide ")
            return False, " L'identifiant de la catégorie est invalide "



        local_id = str(diction['_id'])

        del diction['token']
        del diction['_id']


        new_data['date_update'] = str(datetime.now())
        new_data['update_by'] = str(my_partner['_id'])

        data_cle = {}
        data_cle['partner_owner_recid'] = str(my_partner['recid'])
        data_cle['_id'] = ObjectId(local_id)
        data_cle['valide'] = "1"
        data_cle['locked'] = "0"

        result = MYSY_GV.dbname['class_categorie'].find_one_and_update(
            data_cle,
            {"$set": new_data},
            upsert=False,
            return_document=ReturnDocument.AFTER
        )

        if ("_id" not in result.keys()):
            mycommon.myprint(
                " Impossible de mettre à jour la catégorie de formation (2) ")
            return False, " Impossible de mettre à jour la catégorie de formation (2)  "

        return True, " La catégorie de formation a été correctement mise à jour"

    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        mycommon.myprint(str(inspect.stack()[0][3]) + " -" + str(e) + " - Line : " + str(exc_tb.tb_lineno))
        return False, "  Impossible de mettre à jour la catégorie de formation "


"""
Suppression d'une categorie de formation
regles : 
Si la condition (_id) n'est pas utiliser dans les collection
- myclasss
"""

def Delete_Class_Categorie(diction):
    try:
        diction = mycommon.strip_dictionary(diction)

        """
        Verification des input acceptés
        """
        field_list = ['token', '_id',]

        incom_keys = diction.keys()
        for val in incom_keys:
            if val not in field_list and val.startswith('my_') is False:
                mycommon.myprint(str(
                    inspect.stack()[0][3]) + "  Le champ  '" + val + "' n'est pas autorisé")
                return False, " Les informations fournies sont incorrectes"

        """
        Verification des champs obligatoires
        """
        field_list_obligatoire = ['token', '_id',]

        for val in field_list_obligatoire:
            if val not in diction:
                mycommon.myprint(
                    str(inspect.stack()[0][3]) + " - La valeur '" + val + "' n'est pas presente dans liste ")
                return False, " Les informations fournies sont incorrectes"

        """
        Verification de l'identité et autorisation de l'entité qui
        appelle cette API
        """
        token = ""
        if ("token" in diction.keys()):
            if diction['token']:
                token = diction['token']

        local_status, my_partner = mycommon.Check_Connexion_And_Return_Partner_Data(diction)
        if (local_status is not True):
            return local_status, my_partner


        # Verifier que la codition de paiement existe
        is_existe_class_categorie = MYSY_GV.dbname['class_categorie'].count_documents(
            {'_id': ObjectId(str(diction['_id'])),
             'valide': '1',
             'partner_owner_recid': str(my_partner['recid'])})

        if (is_existe_class_categorie < 0):
            mycommon.myprint(
                str(inspect.stack()[0][3]) + " L'identifiant de la catégorie de formation n'est pas valide ")
            return False, " L'identifiant de la catégorie de formation n'est pas valide  "


        """ 
        Verifier que la categorie de formation n'est pas utilisée (collection : myclass)
        """
        is_categorie_in_myclass = MYSY_GV.dbname['myclass'].count_documents({'partner_owner_recid':my_partner['recid'],
                                                                                                       'valide':'1',
                                                                                                       'class_categorie_id':str(diction['_id'])})


        if( is_categorie_in_myclass > 0 ):
            mycommon.myprint(
                str(inspect.stack()[0][3]) + " Cette catégorie de formation  est utilisée dans "+str(is_categorie_in_myclass)+" formations ")
            return False, " Cette catégorie de formation  est utilisée dans "+str(is_categorie_in_myclass)+" formations "




        delete = MYSY_GV.dbname['class_categorie'].delete_one({'_id': ObjectId(str(diction['_id'])),
                                                      'partner_owner_recid': str(my_partner['recid']),
                                                     }, )



        return True, " La catégorie de formation a été correctement supprimée"

    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        mycommon.myprint(str(inspect.stack()[0][3]) + " -" + str(e) + " - Line : " + str(exc_tb.tb_lineno))
        return False, " Impossible de supprimer la catégorie de formation "



"""
Recuperer la liste des categorie de formation d'un partenaire
"""
def Get_List_Class_Categorie(diction):
    try:
        diction = mycommon.strip_dictionary(diction)

        """
        Verification des input acceptés
        """
        field_list = ['token', ]

        incom_keys = diction.keys()
        for val in incom_keys:
            if val not in field_list  and val.startswith('my_') is False:
                mycommon.myprint(str(
                    inspect.stack()[0][3]) + "  Le champ  '" + val + "' n'existe pas")
                return False, " Les informations fournies sont incorrectes",

        """
        Verification des champs obligatoires
        """
        field_list_obligatoire = ['token', ]
        for val in field_list_obligatoire:
            if val not in diction:
                mycommon.myprint(
                    str(inspect.stack()[0][3]) + " - La valeur '" + val + "' n'est pas presente dans liste ")
                return False, " Les informations fournies sont incorrectes",

        """
        Verification de l'identité et autorisation de l'entité qui
        appelle cette API
        """
        token = ""
        if ("token" in diction.keys()):
            if diction['token']:
                token = diction['token']

        local_status, my_partner = mycommon.Check_Connexion_And_Return_Partner_Data(diction)
        if (local_status is not True):
            return local_status, my_partner

        """
                Clés de mise à jour 
                """
        data_cle = {}
        data_cle['partner_owner_recid'] = str(my_partner['recid'])
        data_cle['valide'] = "1"
        data_cle['locked'] = "0"


        RetObject = []
        val_tmp = 0

        for retval in MYSY_GV.dbname['class_categorie'].find(data_cle).sort([("_id", pymongo.DESCENDING), ]):
            user = retval
            user['id'] = str(val_tmp)
            val_tmp = val_tmp + 1

            if( "class_domaine_id" not in retval.keys() ):
                user['class_domaine_id'] = ""

            RetObject.append(mycommon.JSONEncoder().encode(user))

        return True, RetObject


    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        mycommon.myprint(str(inspect.stack()[0][3]) + " -" + str(e) + " - Line : " + str(exc_tb.tb_lineno))
        return False, " Impossible de récupérer la liste des categories de formation "



"""
Recuperer les données d'une categorie de formtion donnée
"""
def Get_Given_Class_Categorie(diction):
    try:
        diction = mycommon.strip_dictionary(diction)

        """
        Verification des input acceptés
        """
        field_list = ['token', '_id']

        incom_keys = diction.keys()
        for val in incom_keys:
            if val not in field_list  and val.startswith('my_') is False:
                mycommon.myprint(str(
                    inspect.stack()[0][3]) + "  Le champ  '" + val + "' n'existe pas")
                return False, " Les informations fournies sont incorrectes",

        """
        Verification des champs obligatoires
        """
        field_list_obligatoire = ['token', '_id' ]
        for val in field_list_obligatoire:
            if val not in diction:
                mycommon.myprint(
                    str(inspect.stack()[0][3]) + " - La valeur '" + val + "' n'est pas presente dans liste ")
                return False, " Les informations fournies sont incorrectes",

        """
        Verification de l'identité et autorisation de l'entité qui
        appelle cette API
        """
        token = ""
        if ("token" in diction.keys()):
            if diction['token']:
                token = diction['token']

        local_status, my_partner = mycommon.Check_Connexion_And_Return_Partner_Data(diction)
        if (local_status is not True):
            return local_status, my_partner

        """
                Clés de mise à jour 
                """
        data_cle = {}
        data_cle['partner_owner_recid'] = str(my_partner['recid'])
        data_cle['valide'] = "1"
        data_cle['locked'] = "0"
        data_cle['_id'] = ObjectId(str(diction['_id']))


        RetObject = []
        val_tmp = 0

        for retval in MYSY_GV.dbname['class_categorie'].find(data_cle):
            user = retval
            user['id'] = str(val_tmp)
            val_tmp = val_tmp + 1
            RetObject.append(mycommon.JSONEncoder().encode(user))

        return True, RetObject


    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        mycommon.myprint(str(inspect.stack()[0][3]) + " -" + str(e) + " - Line : " + str(exc_tb.tb_lineno))
        return False, " Impossible de récupérer les données de la categorie de formation "


